import { Tooltip, TooltipView } from "@bokehjs/models/ui/tooltip";
import { IterViews } from "@bokehjs/core/build_views";
import * as p from "@bokehjs/core/properties";
import { RadioButtonGroup as bkRadioButtonGroup, RadioButtonGroupView as bkRadioButtonGroupView } from '@bokehjs/models/widgets/radio_button_group';
export declare class RadioButtonGroupView extends bkRadioButtonGroupView {
    model: RadioButtonGroup;
    protected tooltip: TooltipView | null;
    children(): IterViews;
    lazy_initialize(): Promise<void>;
    remove(): void;
    render(): void;
}
export declare namespace RadioButtonGroup {
    type Attrs = p.AttrsOf<Props>;
    type Props = bkRadioButtonGroup.Props & {
        tooltip: p.Property<Tooltip | null>;
    };
}
export interface RadioButtonGroup extends RadioButtonGroup.Attrs {
}
export declare class RadioButtonGroup extends bkRadioButtonGroup {
    properties: RadioButtonGroup.Props;
    __view_type__: RadioButtonGroupView;
    static __module__: string;
    constructor(attrs?: Partial<RadioButtonGroup.Attrs>);
}
